/*-----------------------------------------------------------------------
esurban_table4.do

Stata do file to replicate table 4 and initiate table 6 of the article
'Learning by working in big cities',
by Jorge De la Roca and Diego Puga,
Review of Economic Studies

This calls esurban_table4_boot.do
-----------------------------------------------------------------------*/

* Initialise
clear all
set more off
set matsize 2000
quietly capture log close
log using ../../output/esurban_table4, text replace

* Set number of bootstrap iterations
* (for standard errors of column (1) of table 4, and corresponding results in table 6)
local bstotal = 100
* Set number of parallel batch jobs launched for the bootstrapping
* (this can be adjusted to be as low as bsjobs = 2 depending on processor and memory availability)
local bsjobs = 8
* Set seeds for boostrapping
* (8 random seeds for the pseudorandom number generator were drawn from random.org based on atmospheric noise and hard coded for replicability)
matrix input BSSEEDS = (33856, 529877, 243738, 807939, 540550, 704819, 325745, 678619)

* Define tempfiles
tempfile it_fe
tempfile it_xb
tempfile table_4_r2
tempfile table_4_bootse

* Read panel
use ../../output/mcvl_panel, clear
* Generate extra variables
gen ln_earnings = ln(earnings)
gen experience2 = experience^2
gen tenure2 = tenure^2
gen age2 = age^2
gen uexp_1_2_exp = uexp_1_2 * experience
gen uexp_3_5_exp = uexp_3_5 * experience
gen uexp_1_2_now5 = uexp_1_2 * ua_1_5
gen uexp_1_2_exp_now5 = uexp_1_2_exp * ua_1_5
gen uexp_3_5_now5 = uexp_3_5 * ua_1_5
gen uexp_3_5_exp_now5 = uexp_3_5_exp * ua_1_5
gen uexp_smaller_now5 = uexp_smaller * ua_1_5
gen uexp_smaller_exp_now5 = uexp_smaller * experience * ua_1_5
* Generate variables in table absent from panel
gen ln_size = .
gen city_dummies = .
gen fixed_effects = .
* Generate interactions of estimated fixed-effects and experience variables
gen fe_uexp_1_2 = .
gen fe_uexp_1_2_exp = .
gen fe_uexp_3_5 = .
gen fe_uexp_3_5_exp = .
gen fe_exp = .
gen fe_exp2 = .
gen ln_earnings_fe = .
* Classify individuals based on activity and city size on specific date (May 2007, date = 41)
gen cat_200705 = 0
replace cat_200705 = 1 if date == 41 & ua_1_5 == 0
replace cat_200705 = 2 if date == 41 & ua_1_5 == 1
egen cat = max(cat_200705), by (person_id)
drop cat_200705
label var cat "City size category May 2007"
label define lbl_cat 0 "Not working May 2007"
label define lbl_cat 1 "Working outside 5 biggest cities May 2007", add
label define lbl_cat 2 "Working in 5 biggest cities May 2007", add
label values cat lbl_cat
save ../../output/tmp_panel, replace

* Define mean local experience
summarize uexp_current, meanonly
local mean_exp `r(mean)'

* Define controls
local controls "uexp_1_2 uexp_1_2_exp uexp_3_5 uexp_3_5_exp experience experience2 uexp_1_2_now5 uexp_1_2_exp_now5 uexp_3_5_now5 uexp_3_5_exp_now5 uexp_smaller_now5 uexp_smaller_exp_now5 tenure tenure2 skill_1 skill_2 skill_3 skill_4 contract_fixedterm contract_parttime date_2 - date_72 sector2d_16 - sector2d_95"

* Obtain initial fe as in table 2
xtset person_id date
xtreg ln_earnings ua_id_1 - ua_id_82 `controls', fe cluster(person_id)
xtset, clear
* Generate residuals
predict xb
gen double resid = ln_earnings - xb
* Generate individual fixed-effects
collapse (mean) fe=resid, by (person_id)
* Save as initial fe for iterative procedure
save "`it_fe'", replace
* Save for bootstrapping of fe to be used for table 6 row 2
use ../../output/tmp_panel, clear
merge m:1 person_id using "`it_fe'"
drop _merge
collapse (mean) fe, by (person_id cat)
gen order_id = 0
gen long nident = person_id * 100 + order_id
order person_id order_id nident cat fe
label var order_id "Repeated individual observation (sequential order)"
label var nident "Unique individual (real or virtual) identifier"
label var fe "Worker fixed-effect, homogenous dynamic and static premium (table 2, column 1)"
sort person_id order_id
save ../../output/esurban_table6_fe_row2, replace
forvalues pt = 1(1)`bsjobs' {
  save ../../output/esurban_table6_fe_row2_part`pt', replace
}

* Initialise for loop
local bsno = 0
local itno = 0
local ssd_coef = 100
local rmse = 100
local d_rmse = 100
matrix COEFF = J(220,1,10^6)
* Iterate until convergence
while `ssd_coef' >= 1 | `d_rmse' >= 1 {
  quiet{
    local itno = `itno' + 1
    use ../../output/tmp_panel, clear
    * Recover previous set of fixed-effects
    merge m:1 person_id using "`it_fe'"
    drop _merge
    * Update interactions of estimated fixed-effects and experience variables
    replace fe_uexp_1_2 = fe * uexp_1_2
    replace fe_uexp_1_2_exp = fe * uexp_1_2_exp
    replace fe_uexp_3_5 = fe * uexp_3_5
    replace fe_uexp_3_5_exp = fe * uexp_3_5_exp
    replace fe_exp = fe * experience
    replace fe_exp2 = fe * experience2
    replace ln_earnings_fe = ln_earnings - fe
    * Estimate the value of such interactions and all other coefficients using pooled OLS
    reg ln_earnings_fe ua_id_1 - ua_id_82 fe_uexp_1_2 - fe_exp2 `controls', vce(cluster person_id)
    local rmse_1 = e(rmse)
    matrix COEFFN = e(b)'
    * Predict dependent variable excluding interactions between estimated fixed-effects and experience variables
    preserve
    foreach var of varlist fe_uexp_1_2 - fe_exp2 {
      replace `var' = 0
    }
    predict xb
    keep person_id year month xb
    save `it_xb', replace
    restore
    merge 1:1 person_id year month using `it_xb', keepusing(xb)
    drop _merge
    * Calculate new set of fixed-effects
    gen double fe_numerator = ln_earnings - xb
    gen double fe_denominator = 1 + _b[fe_uexp_1_2]*uexp_1_2 + _b[fe_uexp_1_2_exp]*uexp_1_2_exp + _b[fe_uexp_3_5]*uexp_3_5 + _b[fe_uexp_3_5_exp]*uexp_3_5_exp + _b[fe_exp]*experience + _b[fe_exp2]*experience2 
    replace fe_denominator = 0.001 if fe_denominator == 0
    gen double fe_new = fe_numerator / fe_denominator
    collapse (mean) fe=fe_new, by (person_id)
    save "`it_fe'", replace
    * Check for convergence in estimated coefficients between previous and current iteration
    clear
    set obs 1
    svmat double COEFF
    svmat double COEFFN
    gen double sd_coef = ((COEFFN1 - COEFF1)^2)*10^5
    egen ssdc = sum(sd_coef)
    matrix drop COEFF
    matrix rename COEFFN COEFF
    local ssd_coef "ssdc"
    local d_rmse = (`rmse' - `rmse_1')*10^7
    local rmse = `rmse_1'
  }
  display "Iteration `bsno'/`itno'"
  display "`c(current_date)', `c(current_time)'"
  display "SSD Coef.:" `ssd_coef'
  display "D RMSE: " `d_rmse'
  display ""
}
* Save last estimated fixed-effects (after convergence)
use "`it_fe'", clear
label var fe "Worker fixed-effect, heterogeneous dynamic and static premium (table 4, column 1)"
save ../../output/esurban_table4_fe, replace
* Save for bootstrapping of fe to be used for table 6 row 1
use ../../output/tmp_panel, clear
merge m:1 person_id using ../../output/esurban_table4_fe
drop _merge
preserve
collapse (mean) fe, by (person_id cat)
gen order_id = 0
gen long nident = person_id * 100 + order_id
order person_id order_id nident cat fe
label var order_id "Repeated individual observation (sequential order)"
label var nident "Unique individual (real or virtual) identifier"
sort person_id order_id
save ../../output/esurban_table6_fe_row1, replace
forvalues pt = 1(1)`bsjobs' {
  save ../../output/esurban_table6_fe_row1_part`pt', replace
}
restore

* Column (1), estimate the regression to be reported using last estimated fixed-effects (after convergence) and save coefficients
replace fe_uexp_1_2 = fe * uexp_1_2
replace fe_uexp_1_2_exp = fe * uexp_1_2_exp
replace fe_uexp_3_5 = fe * uexp_3_5
replace fe_uexp_3_5_exp = fe * uexp_3_5_exp
replace fe_exp = fe * experience
replace fe_exp2 = fe * experience2
replace ln_earnings_fe = ln_earnings - fe
reg ln_earnings_fe ua_id_1 - ua_id_82 fe_uexp_1_2 - fe_exp2 `controls', vce(cluster person_id)
est2vec esurban_table4, e(r2) name(a)
matsave esurban_table4_b, replace saving path(../../output/)
preserve
use ../../output/esurban_table4_b, clear
erase ../../output/esurban_table4_b.dta
rename _rowname coeff_var
rename a boot0
gen coeff_no = _n
order coeff_no coeff_var boot0
save ../../output/esurban_table4_coeff, replace
forvalues pt = 1(1)`bsjobs' {
  save ../../output/esurban_table4_coeff_part`pt', replace
}
restore
est2vec esurban_table4, replace e(r2) vars(ln_size city_dummies fixed_effects uexp_1_2 uexp_1_2_exp uexp_3_5 uexp_3_5_exp experience experience2 fe_uexp_1_2 fe_uexp_1_2_exp fe_uexp_3_5 fe_uexp_3_5_exp fe_exp fe_exp2 uexp_1_2_now5 uexp_1_2_exp_now5 uexp_3_5_now5 uexp_3_5_exp_now5 uexp_smaller_now5 uexp_smaller_exp_now5) name(a)

* Combine 1st stage coefficients for city indicators with medium term premium and log city size
matrix COEFF = e(b)'
gen num_obs = 1
collapse (sum) num_obs, by (ua_id)
svmat double COEFF, names(col)
replace y1 = 0 if ua_id == 83
drop if missing(ua_id)
rename y1 premium_initial_het
gen uexp_1_2 = _b[uexp_1_2] * `mean_exp' if ua_id == 1 | ua_id == 2
gen uexp_1_2_exp = _b[uexp_1_2_exp] * `mean_exp'^2 if ua_id == 1 | ua_id == 2
gen uexp_3_5  = _b[uexp_3_5] * `mean_exp' if ua_id == 3 | ua_id == 4 | ua_id == 8
gen uexp_3_5_exp = _b[uexp_3_5_exp] * `mean_exp'^2 if ua_id == 3 | ua_id == 4 | ua_id == 8
gen uexp_1_2_now5 = _b[uexp_1_2_now5] * `mean_exp' if ua_id == 1 | ua_id == 2
gen uexp_1_2_exp_now5 = _b[uexp_1_2_exp_now5] * `mean_exp'^2 if ua_id == 1 | ua_id == 2
gen uexp_3_5_now5 = _b[uexp_3_5_now5] * `mean_exp' if ua_id == 3 | ua_id == 4 | ua_id == 8
gen uexp_3_5_exp_now5 = _b[uexp_3_5_exp_now5] * `mean_exp'^2 if ua_id == 3 | ua_id == 4 | ua_id == 8
egen premium_exp = rowtotal(uexp_1_2 - uexp_3_5_exp_now5)
gen premium_medterm_het = premium_initial_het + premium_exp
drop premium_exp uexp_*
merge 1:1 ua_id using ../../otherdata/ua_size, keepusing(size_pop10k)
drop if _merge == 2
drop _merge num_obs
gen ln_size = ln(size_pop10k)
drop size_pop10k

* Column (2), 2nd stage initial premium
reg premium_initial_het ln_size, robust
est2vec esurban_table4, addto(esurban_table4) name(b)

* Column (3), 2nd stage medium premium
reg premium_medterm_het ln_size, robust
est2vec esurban_table4, addto(esurban_table4) name(c)

* Replace R2 in column (1) of table 4 by within
est2texdp esurban_table4, label preserve replace path(../../output/) levels(90 95 99) mark(stars) digits(4) leadzero flexible(0) fancy
matsave esurban_table4_b, replace saving path(../../output/) 
matsave esurban_table4_e, replace saving path(../../output/) 
matrix esurban_table4_base_se = esurban_table4_se
matsave esurban_table4_base_se, replace saving path(../../output/) 
use ../../output/tmp_panel, clear
merge m:1 person_id using ../../output/esurban_table4_fe
drop _merge
replace fe_uexp_1_2 = fe * uexp_1_2
replace fe_uexp_1_2_exp = fe * uexp_1_2_exp
replace fe_uexp_3_5 = fe * uexp_3_5
replace fe_uexp_3_5_exp = fe * uexp_3_5_exp
replace fe_exp = fe * experience
replace fe_exp2 = fe * experience2
replace ln_earnings_fe = ln_earnings - fe
xtset person_id date
xtreg ln_earnings_fe ua_id_1 - ua_id_82 fe_uexp_1_2 - fe_exp2 `controls', vce(cluster person_id)
xtset, clear
clear
set obs 1
gen r2_within = e(r2_w)
gen _rowname = "e(r2)"
save `table_4_r2', replace
use ../../output/esurban_table4_e, clear
gen order = _n
merge 1:1 _rowname using `table_4_r2'
drop _merge
replace a = r2_within if _rowname == "e(r2)"
sort order
drop order r2_within
save ../../output/esurban_table4_e, replace

* Obtain baseline fe to be used for table 6 row 3
use ../../output/tmp_panel, clear
xtset person_id date
xtreg ln_earnings ua_id_1 - ua_id_82 age age2 date_2 - date_72 sector2d_16 - sector2d_95, fe cluster(person_id)
xtset, clear
* Generate residuals
predict xb
gen double resid = ln_earnings - xb
* Generate individual fixed-effects
collapse (mean) fe=resid, by (person_id)
* Save for bootstrapping of fe to be used for table 6 row 3
save "`it_fe'", replace
use ../../output/tmp_panel, clear
merge m:1 person_id using "`it_fe'"
drop _merge
collapse (mean) fe, by (person_id cat)
gen order_id = 0
gen long nident = person_id * 100 + order_id
order person_id order_id nident cat fe
label var order_id "Repeated individual observation (sequential order)"
label var nident "Unique individual (real or virtual) identifier"
label var fe "Worker fixed-effects, static premium (Combes et al., 2012)"
sort person_id order_id
save ../../output/esurban_table6_fe_row3, replace
forvalues pt = 1(1)`bsjobs' {
  save ../../output/esurban_table6_fe_row3_part`pt', replace
}

* Obtain baseline log earnings distribution to be used for table 6
use ../../output/tmp_panel, clear
keep if date == 41
rename ln_earnings w
keep person_id cat w
gen order_id = 0
gen long nident = person_id * 100 + order_id
order person_id order_id nident cat w
label var order_id "Repeated individual observation (sequential order)"
label var nident "Unique individual (real or virtual) identifier"
label var w "Log earnings in May 2007"
sort person_id order_id
save ../../output/esurban_table6_w, replace
forvalues pt = 1(1)`bsjobs' {
  save ../../output/esurban_table6_w_part`pt', replace
}
clear

* Ensure number of seeds matches bsjobs  
if `bsjobs' <= colsof(BSSEEDS) {
  matrix BSSEEDS = BSSEEDS[1,1..`bsjobs']
}
else {
  local extrabsseeds = `bsjobs' - colsof(BSSEEDS)
  forvalues i = 1(1)`extrabsseeds' {
    matrix BSSEEDS = BSSEEDS, runiformint(1,10^6)
  }
}
local bsseeds = string(el(BSSEEDS,1,1))
forvalues i = 2(1)`bsjobs' {
local element = el(BSSEEDS,1,`i')
  local bsseeds = "`bsseeds' " + string(el(BSSEEDS,1,`i'))
}

* Bootstrap for standard errors of column (1) of table 4 and table 6
* (split into `bsjobs' jobs run in parallel)
parallel setclusters `bsjobs'
parallel do esurban_table4_boot.do, nodata seeds("`bsseeds'")

* Combine bootstrapped fixed effects from parallel jobs and save for table 6
forvalues rowi = 1(1)3 {
  use ../../output/esurban_table6_fe_row`rowi'_part1, clear
  forvalues pt = 2(1)`bsjobs' {
    merge 1:1 nident using ../../output/esurban_table6_fe_row`rowi'_part`pt'
    drop _merge
  }
  order fe*, sequential
  order person_id order_id nident cat
  forvalues i = 1(1)`bstotal' {
    label var fe`i' "Worker fixed-effect: iteration `i'"
  }
  compress
  save ../../output/esurban_table6_fe_row`rowi', replace
}
use ../../output/esurban_table6_w_part1, clear
forvalues pt = 2(1)`bsjobs' {
  merge 1:1 nident using ../../output/esurban_table6_w_part`pt'
  drop _merge
}
order w*, sequential
order person_id order_id nident cat
forvalues i = 1(1)`bstotal' {
  label var w`i' "Log earnings in May 2007: iteration `i'"
}
compress
sort person_id order_id
save ../../output/esurban_table6_w, replace

* Combine bootstrapped coefficients from parallel jobs
use ../../output/esurban_table4_coeff_part1, clear
forvalues pt = 2(1)`bsjobs' {
  merge 1:1 coeff_no using ../../output/esurban_table4_coeff_part`pt'
  drop _merge
}
* Replace standard errors in column (1) of table 4 by bootstrapped standard errors
order boot*, sequential
order coeff_var coeff_no
label var coeff_var "Variable name"
label var coeff_no "Variable order in table 4 column (1) regression"
label var boot0 "Estimated coefficient in table 4 column (1) regression"
forvalues i = 1(1)`bstotal' {
  label var boot`i' "Estimated coefficient: iteration `i'"
}
compress
save ../../output/esurban_table4_coeff, replace
forvalues pt = 1(1)`bsjobs' {
  forvalues rowi = 1(1)3 {
    erase ../../output/esurban_table6_fe_row`rowi'_part`pt'.dta
  }
  erase ../../output/esurban_table6_w_part`pt'.dta
  erase ../../output/esurban_table4_coeff_part`pt'.dta
}
erase ../../output/tmp_panel.dta
egen a_boot = rsd(boot1 - boot`bstotal')
rename coeff_var _rowname
keep _rowname coeff_no a_boot
save `table_4_bootse', replace
use ../../output/esurban_table4_base_se, clear
gen corder = _n
merge 1:1 _rowname using `table_4_bootse'
drop _merge
drop if missing(a)
replace a_boot = -999 if missing(a_boot)
sort corder
replace a = a_boot
drop corder a_boot coeff_no
save ../../output/esurban_table4_se, replace

* Produce LaTeX table
clear
clear matrix
set obs 1
gen ln_size = .
gen city_dummies = .
gen fixed_effects = .
gen uexp_1_2 = .
gen uexp_1_2_exp = .
gen uexp_3_5 = .
gen uexp_3_5_exp = .
gen experience = .
gen experience2 = .
gen fe_uexp_1_2 = .
gen fe_uexp_1_2_exp = .
gen fe_uexp_3_5 = .
gen fe_uexp_3_5_exp = .
gen fe_exp = .
gen fe_exp2 = .
gen uexp_1_2_now5 = .
gen uexp_1_2_exp_now5 = .
gen uexp_3_5_now5 = .
gen uexp_3_5_exp_now5 = .
gen uexp_smaller_now5 = .
gen uexp_smaller_exp_now5 = .
gen uexp_12_15 = .
gen uexp_12_exp_15 = .
gen uexp_35_15 = .
gen uexp_35_exp_15 = .
gen uexp_676_15 = .
gen uexp_676_exp_15 = .
label var ln_size "Log city size"
label var city_dummies "City indicators"
label var fixed_effects "Worker fixed-effects"
label var uexp_1_2 "Experience 1$^\text{st}$-2$^\text{nd}$ biggest cities"
label var uexp_1_2_exp "Experience 1$^\text{st}$-2$^\text{nd}$ biggest cities $\times$ experience"
label var uexp_3_5 "Experience 3$^\text{rd}$-5$^\text{th}$ biggest cities"
label var uexp_3_5_exp "Experience 3$^\text{rd}$-5$^\text{th}$ biggest cities $\times$ experience"
label var experience "Experience"
label var experience2 "Experience$^2$"
label var fe_uexp_1_2 "Experience 1$^\text{st}$-2$^\text{nd}$ biggest $\times$ worker fixed-effect"
label var fe_uexp_1_2_exp "Experience 1$^\text{st}$-2$^\text{nd}$ biggest $\times$ experience $\times$ worker fixed-effect"
label var fe_uexp_3_5 "Experience 3$^\text{rd}$-5$^\text{th}$ biggest cities $\times$ worker fixed-effect"
label var fe_uexp_3_5_exp "Experience 3$^\text{rd}$-5$^\text{th}$ biggest $\times$ experience $\times$ worker fixed-effect"
label var fe_exp "Experience $\times$ worker fixed-effect"
label var fe_exp2 "Experience$^2 \times$ worker fixed-effect"
label var uexp_1_2_now5 "Experience 1$^\text{st}$-2$^\text{nd}$ biggest $\times$ now in 5 biggest"
label var uexp_1_2_exp_now5 "Experience 1$^\text{st}$-2$^\text{nd}$ biggest $\times$ experience $\times$ now in 5 biggest"
label var uexp_3_5_now5 "Experience 3$^\text{rd}$-5$^\text{th}$ biggest $\times$ now in 5 biggest"
label var uexp_3_5_exp_now5 "Experience 3$^\text{rd}$-5$^\text{th}$ biggest $\times$ experience $\times$ now in 5 biggest"
label var uexp_smaller_now5 "Experience outside 5 biggest $\times$ now in 5 biggest"
label var uexp_smaller_exp_now5 "Experience outside 5 biggest $\times$ experience $\times$ now in 5 biggest"
matload esurban_table4_b, saving path(../../output/) 
matload esurban_table4_se, saving path(../../output/) 
matload esurban_table4_e, saving path(../../output/) 
erase ../../output/esurban_table4_b.dta
erase ../../output/esurban_table4_se.dta
erase ../../output/esurban_table4_base_se.dta
erase ../../output/esurban_table4_e.dta
erase ../../output/esurban_table4_fe.dta
est2texdp esurban_table4, label preserve replace path(../../output/) levels(90 95 99) mark(stars) digits(4) leadzero flexible(0) fancy
erase ../../output/esurban_table4_tbl.dta

log close

