/*-----------------------------------------------------------------------
esurban_table6.do

Stata do file to replicate table 6 of the article
'Learning by working in big cities',
by Jorge De la Roca and Diego Puga,
Review of Economic Studies

This requires esurban_table4.do to be run before
-----------------------------------------------------------------------*/

* Initialise
clear all
set more off
set matsize 2000
quietly capture log close
log using ../../output/esurban_table6, text replace

* Path to SAS
local SASpath "/usr/local/SASHome/SASFoundation/9.3/sas"

* Number of bootstrap iterations
local bstotal = 100

* Get current working directory (to be passed on to SAS)
local basedir `c(pwd)'

* Define tempfiles
tempfile tmpf

* Use SAS to estimate shift and dilation for each bootstrap iteration of worker fixed-effects and log earnings
forvalues rowi = 1(1)3 {
  local sasvar "fe"
  local sasdata "esurban_table6_fe_row`rowi'"
  local sasvars "`sasvar'"
  forvalues bsno = 1(1)`bstotal' {
    local sasvars "`sasvars' `sasvar'`bsno'"
  }
  stcmd "../../output/`sasdata'.dta" "`sasdata'.sas7bdat" -y
  shell `SASpath' -sysin "`basedir'/cdgprmainprogram.sas" -initstmt "%let basedir=`basedir'; %let data=`sasdata'; %let variables=`sasvars';"
  erase "`sasdata'.sas7bdat"
  stcmd "results`sasdata'.sas7bdat" "../../output/results`sasdata'.dta" -y
  erase "results`sasdata'.sas7bdat"
  erase cdgprmainprogram.log
  use "../../output/results`sasdata'", clear
  rename criteria cr
  rename a A
  rename s S
  rename d D
  gen int spec = `rowi'
  gen boot_it = 0
  forvalues sb = 1(1)`bstotal' {
    replace boot_it = `sb' if name == "`sasvar'`sb'"
  }
  save "../../output/sasresults_fe_`rowi'", replace
  erase "../../output/results`sasdata'.dta"
}

local sasvar "w"
local sasdata "esurban_table6_w"
local sasvars "`sasvar'"
forvalues bsno = 1(1)`bstotal' {
  local sasvars "`sasvars' `sasvar'`bsno'"
}
stcmd "../../output/`sasdata'.dta" "`sasdata'.sas7bdat" -y
shell `SASpath' -sysin "`basedir'/cdgprmainprogram.sas" -initstmt "%let basedir=`basedir'; %let data=`sasdata'; %let variables=`sasvars';"
erase "`sasdata'.sas7bdat"
stcmd "results`sasdata'.sas7bdat" "../../output/results`sasdata'.dta" -y
erase "results`sasdata'.sas7bdat"
erase cdgprmainprogram.log
use "../../output/results`sasdata'", clear
rename criteria cr
rename a A
rename s S
rename d D
gen int spec = 4
gen boot_it = 0
forvalues sb = 1(1)`bstotal' {
  replace boot_it = `sb' if name == "`sasvar'`sb'"
}
save "../../output/sasresults_w", replace
erase "../../output/results`sasdata'.dta"

* Combine results for different specifications
use "../../output/sasresults_fe_1", clear
append using "../../output/sasresults_fe_2"
append using "../../output/sasresults_fe_3"
append using "../../output/sasresults_w"
forvalues rowi = 1(1)3 {
  erase "../../output/sasresults_fe_`rowi'.dta"
}
erase "../../output/sasresults_w.dta"
gen spec_description = ""
replace spec_description = "Worker fixed-effect, heterogeneous dynamic and static premium (table 4, column 1)" if spec == 1
replace spec_description = "Worker fixed-effects, homogenous dynamic and static premium (table 2, column 1)" if spec == 2
replace spec_description = "Worker fixed-effects, static premium (Combes et al., 2012)" if spec == 3
replace spec_description = "Log earnings" if spec == 4
sort spec
preserve
keep if (boot_it==0 & shift==0 & dilation==0 & truncation==0)
rename n1t n1
rename n2t n2
keep spec cr obs n1 n2
rename cr cr_base
sort spec
save "`tmpf'", replace
restore
drop if (shift==0 & dilation==0 & truncation==0)
merge m:1 spec using "`tmpf'"
drop _merge
keep if (shift==1 & dilation==1 & truncation==0)
save "../../output/esurban_table6_sasresults", replace

* Compute standard errors and significance levels
preserve
drop if boot_it == 0
foreach var in "A" "D" "S" {
  by spec, sort: egen `var'_se = sd(`var') 
}
collapse (first) A_se D_se S_se, by(spec)
save "`tmpf'", replace
restore
keep if boot_it==0
merge m:1 spec using "`tmpf'"
drop _merge
sort spec
gen A_T = abs(A/A_se)
gen D_T = abs((D-1)/D_se)
gen S_T = abs(S/S_se)
foreach var in "A" "D" "S" {
  gen `var'_sig = .
  replace `var'_sig = 10 if `var'_T >= 1.645
  replace `var'_sig = 5 if `var'_T >= 1.96
  replace `var'_sig = 1 if `var'_T >= 2.576
}
keep spec spec_description A A_se A_sig D D_se D_sig S S_se S_sig cr cr_base R2 obs n1 n2
order spec spec_description A A_se A_sig D D_se D_sig S S_se S_sig cr cr_base R2 obs n1 n2

* Create extra table row with standard errors and significance
expand 2, generate(serow)
format A A_se D D_se S S_se %9.4f
format cr cr_base %4.1g
format R2 %9.4f
format obs n1 n2 %9.0fc
tostring A A_se D D_se S S_se R2 obs n1 n2, replace force usedisplayformat
foreach var in "A" "D" "S" {
  replace `var' = "\raisebox{.7ex}[0pt]{\scriptsize (" + `var'_se + ")$^{~***}$}" if serow == 1 & `var'_sig == 1
  replace `var' = "\raisebox{.7ex}[0pt]{\scriptsize (" + `var'_se + ")$^{~**}$}" if serow == 1 & `var'_sig == 5
  replace `var' = "\raisebox{.7ex}[0pt]{\scriptsize (" + `var'_se + ")$^{~*}$}" if serow == 1 & `var'_sig == 10
  replace `var' = "\raisebox{.7ex}[0pt]{\scriptsize (" + `var'_se + ")}" if serow == 1 & `var'_sig == .
}
drop *_se *_sig
replace R2 = "" if serow == 1
replace cr = . if serow == 1
replace cr_base = . if serow == 1
replace obs = "" if serow == 1
replace n1 = "" if serow == 1
replace n2 = "" if serow == 1
replace spec_description = "" if serow == 1
* Generate extra LaTeX formatting
expand 2, generate(rulerow)
drop if rulerow == 1 & serow == 0
replace serow = 2 if rulerow == 1
drop rulerow
sort spec serow
drop if serow == 2 & spec == 4
replace spec_description = "\midrule" if serow == 2
foreach var in "A" "D" "S" {
  replace `var' = "" if serow == 2
}
gen str1 ampersand = "&"
replace ampersand = "" if serow == 2
gen str4 slash = "\\"
replace slash = "" if serow == 2
replace spec_description = "Worker fixed-effects, heterogeneous dynamic" if spec == 1 & serow == 0
replace spec_description = "and static premium  (table \ref{tab:dynamic_heterog}, column 1)" if spec == 1 & serow == 1
replace spec_description = "Worker fixed-effects, homogenous dynamic" if spec == 2 & serow == 0
replace spec_description = "and static premium (table \ref{tab:dynamic}, column 1)" if spec == 2 & serow == 1
replace spec_description = "Worker fixed-effects, static premium" if spec == 3 & serow == 0
replace spec_description = "(Combes et al., 2012\textit{b})" if spec == 3 & serow == 1
replace spec_description = "Log earnings" if spec == 4 & serow == 0
replace spec_description = "" if spec == 4 & serow == 1
sort spec serow
* Produce LaTeX table
outsheet spec_description ampersand A ampersand D ampersand cr_base ampersand R ampersand obs slash using ../../output/esurban_table6.tex, nonames noquote replace

log close
